var net = require('net');
var hf = require('./helpFunctions');

// creates the server
var server = net.createServer();

var clients = [];

//emitted when server closes ...not emitted until all connections closes.
server.on('close',function(){
    console.log('Server closed !');
});

// emitted when new client connects
server.on('connection',function(socket){
    // Identify this client
    socket.name = socket.remoteAddress + ":" + socket.remotePort;
    socket.mapId = "null";
    clients.push(socket);

    // Send a nice welcome message and announce
    socket.write("Welcome " + socket.name + "\n");
    broadcast(socket.name + " joined the chat\n", socket);

//this property shows the number of characters currently buffered to be written. (Number of characters is approximately equal to the number of bytes to be written, but the buffer may contain strings, and the strings are lazily encoded, so the exact number of bytes is not known.)
//Users who experience large or growing bufferSize should attempt to "throttle" the data flows in their program with pause() and resume().

    console.log('Buffer size : ' + socket.bufferSize);

    console.log('---------server details -----------------');

    var address = server.address();
    var port = address.port;
    var family = address.family;
    var ipaddr = address.address;
    console.log('Server is listening at port' + port);
    console.log('Server ip :' + ipaddr);
    console.log('Server is IP4/IP6 : ' + family);

    var lport = socket.localPort;
    var laddr = socket.localAddress;
    console.log('Server is listening at LOCAL port' + lport);
    console.log('Server LOCAL ip :' + laddr);

    console.log('------------remote client info --------------');

    var rport = socket.remotePort;
    var raddr = socket.remoteAddress;
    var rfamily = socket.remoteFamily;

    console.log('REMOTE Socket is listening at port' + rport);
    console.log('REMOTE Socket ip :' + raddr);
    console.log('REMOTE Socket is IP4/IP6 : ' + rfamily);

    console.log('--------------------------------------------')
//var no_of_connections =  server.getConnections(); // sychronous version
    server.getConnections(function(error,count){
        console.log('Number of concurrent connections to the server : ' + count);
    });

    socket.setEncoding('utf8');

    socket.setTimeout(900000,function(){
        // called after timeout -> same as socket.on('timeout')
        // it just tells that soket timed out => its ur job to end or destroy the socket.
        // socket.end() vs socket.destroy() => end allows us to send final data and allows some i/o activity to finish before destroying the socket
        // whereas destroy kills the socket immediately irrespective of whether any i/o operation is goin on or not...force destry takes place
        console.log('Socket timed out');
    });

    socket.on('data',function(data){
        console.log('Data sent to server : ' + data);

        if(hf.isJsonString(data)) {
            let url;
            if (hf.checkIfRequestHasVote(data)) {
                url = 'http://localhost:8080/setVote';
            } else {
                url ='http://localhost:8080/getMapInfo';
            }
            hf.sendRequest(url, data, res => {
                if (res) {
                    const req = JSON.parse(data);
                    hf.addMapToClient(clients, socket, req['mapId'], newClientList => {
                        clients = newClientList.slice();
                        socket.write(JSON.stringify(res.data) + '\n');
                        if (hf.checkIfRequestHasVote(data)) {
                            sendToAllClientsAtSameMap(res, req['mapId'], socket);
                        }
                    });
                    //var is_kernel_buffer_full = socket.write(JSON.stringify(res.data) + '\n');
                    //if(is_kernel_buffer_full){
                    //    console.log('Data was flushed successfully from kernel buffer i.e written successfully!');
                    //} else {
                    //    socket.pause();
                    //}
                } else {
                    console.log("Res was null");
                }
            });
        }
    });

    socket.on('drain',function(){
        console.log('write buffer is empty now .. u can resume the writable stream');
        socket.resume();
    });

    socket.on('error',function(error){
        console.log('Error : ' + error);
    });

    socket.on('timeout',function(){
        console.log('Socket timed out !');
        socket.end('Timed out!');
        // can call socket.destroy() here too.
    });

    socket.on('end',function(data){
        console.log('Socket ended from other end!');
        console.log('End data : ' + data);
    });

    socket.on('close',function(error){
        var bread = socket.bytesRead;
        var bwrite = socket.bytesWritten;
        console.log('Bytes read : ' + bread);
        console.log('Bytes written : ' + bwrite);
        console.log('Socket closed!');
        if(error){
            console.log('Socket was closed coz of transmission error');
        }
    });

    setTimeout(function(){
        var isdestroyed = socket.destroyed;
        console.log('Socket destroyed:' + isdestroyed);
        socket.destroy();
    },1200000);

    // Send a message to all clients
    function broadcast(message, sender) {
        clients.forEach(function (client) {
            // Don't want to send it to sender
            if (client === sender) return;
            client.write(message);
        });
        // Log it to the server output too
        process.stdout.write(message)
    }

    // Send a message to all clients playing the same map
    function sendToAllClientsAtSameMap(res, mapId, currentClient) {
        console.log("JSON: ", JSON.stringify(res.data));
        clients.forEach(function (client) {
            const resJson = res.data;
            delete resJson['vote'];
            if (client.mapId === mapId && client.name !== currentClient) client.write(JSON.stringify(resJson) + '\n');
        });
        console.log("Write Message to all at " + mapId +": " + JSON.stringify(res.data));
    }

});

// emits when any error occurs -> calls closed event immediately after this.
server.on('error',function(error){
    console.log('Error: ' + error);
});

//emits when server is bound with server.listen
server.on('listening',function(){
    console.log('Server is listening!');
});

server.maxConnections = 500;

var HOST = '0.0.0.0'; // bind to all interfaces, not just loopback
var PORT = 3000;
//static port allocation
server.listen(PORT, HOST);


// for dyanmic port allocation
/*server.listen(function(){
    var address = server.address();
    var port = address.port;
    var family = address.family;
    var ipaddr = address.address;
    console.log('Server is listening at port' + port);
    console.log('Server ip :' + ipaddr);
    console.log('Server is IP4/IP6 : ' + family);
});



var islistening = server.listening;

if(islistening){
    console.log('Server is listening');
}else{
    console.log('Server is not listening');
}

setTimeout(function(){
    server.close();
},5000000);
*/

//---------------------client----------------------

// creating a custom socket client and connecting it....
/*var client  = new net.Socket();
client.connect({
    host: 'm.willers.digital',
    port: 80
});

client.on('connect',function(){
    console.log('Client: connection established with server');

    console.log('---------client details -----------------');
    var address = client.address();
    var port = address.port;
    var family = address.family;
    var ipaddr = address.address;
    console.log('Client is listening at port' + port);
    console.log('Client ip :' + ipaddr);
    console.log('Client is IP4/IP6 : ' + family);


    // writing data to server
    client.write('hello from client');

});

client.setEncoding('utf8');

client.on('data',function(data){
    console.log('Data from server:' + data);
});

/*setTimeout(function(){
    client.end('Bye bye server');
},5000);
*/
//NOTE:--> all the events of the socket are applicable here..in client...


// -----------------creating client using net.connect instead of custom socket-------

// server creation using net.connect --->
// u can also => write the below code in seperate js file
// open new node instance => and run it...

// 87.78.129.86  | localhost

/*const testClient = net.connect({host: '87.78.129.86', port: 3201}, () => {
    console.log('connected to server!');
    //testClient.write('Hello World Client!\r\n');
    testClient.write(JSON.stringify({"mapId":"7fsfRSUCQ7YwfBEdRk_GivW6qzj","userId":"aKBNg9fbReqLedLiz1KFfA","name":"Summer 2021 - 01"}));
    //test(testClient);
});

testClient.on('data', (data) => {
    console.log(data.toString());
    //testClient.end();
});
/*testClient.on('end', () => {
    console.log('disconnected from server');
});
//*/

/*async function test(clients) {
    let i = 0;
    while (true) {
        await new Promise(resolve => setTimeout(resolve, 5000));
        i = i+1;
        clients.write("Test: "+ i);
    }
}


// ---------------------------------------------- //


/*var client = new net.Socket();
client.connect(8080, 'm.willers.digital', function() {
    console.log('Connected');
    client.write('Hello, server! Love, Client.');
});

client.on('data', function(data) {
    console.log('Received: ' + data);
    client.destroy(); // kill client after server's response
});

client.on('close', function() {
    console.log('Connection closed');
});*/
